<?php
/*
* スケジュールプラグイン
* 管理コントローラー
*
* PHP 5.4.x
*
* @copyright    Copyright (c) hiniarata co.ltd
* @link         https://hiniarata.jp
* @package      Schedule Plugin Project
* @since        ver.0.9.0
*/

/**
* スケジュール　管理コントローラー
*
* @package baser.plugins.schedule
*/
class ScheduleConfigsController extends ScheduleAppController {

  /**
  * クラス名
  *
  * @var string
  * @access public
  */
  public $name = 'ScheduleConfigs';

  /**
  * コンポーネント
  *
  * @var array
  * @access public
  */
  public $components = array('BcAuth', 'Cookie', 'BcAuthConfigure', 'Security');

  /**
  * モデル
  *
  * @var array
  * @access public
  */
  public $uses = array('Schedule.Schedule', 'Schedule.ScheduleCategory', 'Schedule.ScheduleSetting');

  /**
  * ぱんくずナビ
  *
  * @var string
  * @access public
  */
  public $crumbs = array(
  array('name' => 'プラグイン管理', 'url' => array('plugin' => '', 'controller' => 'plugins', 'action' => 'index')),
  array('name' => 'スケジュール管理', 'url' => array('controller' => 'schedule_configs', 'action' => 'index'))
  );

  /**
  * サブメニューエレメント
  *
  * @var array
  * @access public
  */
  public $subMenuElements = array('schedule');

  /**
  * beforeFilter
  *
  * @return	void
  * @access 	public
  */
  public function beforeFilter() {
    parent::beforeFilter();
    //CSRF対策（トークンの有効期限延長）
    $this->Security->csrfCheck = true;
    $this->Security->csrfExpires = '+1 hour';
  }

  /**
  * [ADMIN] 一覧表示
  *
  * @return void
  */
  public function admin_index() {
    // 画面表示情報設定
    $default = array(
      'named' => array(
        'num' => $this->siteConfigs['admin_list_num']
      )
    );
    $this->setViewConditions('Schedule', array('default' => $default));

    //データの取得
    // 並び替えモードの場合は、強制的にsortフィールドで並び替える
      if (!empty($this->passedArgs['sortmode'])) {
        $this->passedArgs['sort'] = 'sort';
        $this->passedArgs['direction'] = 'asc';
        $this->set('sortmode', true);
        //設定
        $this->paginate = array(
            'fields' => array(),
            'order' => 'Schedule.sort',
            'limit' => $this->passedArgs['num']
        );
      } else {
        $this->set('sortmode', false);
        //設定
        $this->paginate = array(
            'fields' => array(),
            'order' => 'Schedule.id DESC',
            'limit' => $this->passedArgs['num']
        );
      }
    /* 表示設定 */
    $this->set('schedule', $this->paginate('Schedule'));
    $this->pageTitle = 'スケジュール 一覧';
    //$this->help = 'schedule_configs_index';
  }


  /**
  * [ADMIN] 新規登録
  *
  * @return void
  */
  public function admin_add() {
    /* 設定情報の取得 */
    $setting = $this->ScheduleSetting->find('first');

    //インストール直後など、値が空の場合に備えデフォルト値を付けておく。
    if (empty($setting)) {
      $setting['ScheduleSetting']['editor_type'] = 'ckeditor';
      $setting['ScheduleSetting']['image01'] = 0;
      $setting['ScheduleSetting']['image02'] = 0;
      $setting['ScheduleSetting']['image03'] = 0;
      $setting['ScheduleSetting']['image04'] = 0;
      $setting['ScheduleSetting']['map'] = 0;
    }

    //カテゴリ情報の取得
    $categories = $this->ScheduleCategory->find('list', array('conditions' => array(
      'ScheduleCategory.publish' => 1
    )));

    //データ保存処理
    if (!empty($this->request->data)) {
      //画像の最大値を確認する
      if (!empty($setting['ScheduleSetting']['max_width'])) {
        $maxWidth = $setting['ScheduleSetting']['max_width'];
      } else {
        $maxWidth = null;
      }
      if (!empty($setting['ScheduleSetting']['max_height'])) {
        $maxHeight = $setting['ScheduleSetting']['max_height'];
      } else {
        $maxHeight = null;
      }

      //画像保存処理実行
      $insertData = $this->Schedule->savePostImg($this->request->data, $maxWidth, $maxHeight);
      $this->Schedule->create();

      //現在の最大ソート番号を取得する。
      $maxSort = $this->Schedule->find('first', array(
                      'fields' => 'MAX(sort)'
                  ));
      //次回のソート番号
      $nextCount = $maxSort[0]['MAX(sort)'] +1;
      $insertData['Schedule']['sort'] = $nextCount;

      //ポストデータの保存
      if ($this->Schedule->save($insertData)) {
        $this->setMessage('スケジュール「'. h($this->request->data['Schedule']['title']) .'」を登録しました。', false, true);
        $this->redirect(array('action' => 'index'));
      } else {
        $this->setMessage('入力エラーです。内容を修正してください。', true);
      }
    }

    //エディタ設定
    $editorOptions = array('editorDisableDraft' => true);
    if (!empty($this->siteConfigs['editor_styles'])) {
      App::uses('CKEditorStyleParser', 'Vendor');
      $CKEditorStyleParser = new CKEditorStyleParser();
      $editorStyles = array('default' => $CKEditorStyleParser->parse($this->siteConfigs['editor_styles']));
      $editorOptions = array_merge($editorOptions, array(
        'editorStylesSet' => 'simple',
        'editorStyles' => $editorStyles
      ));
    }
    $this->set('editorOptions', $editorOptions);

    /* 表示設定 */
    $this->pageTitle = 'スケジュール 新規登録';
    $this->set('setting', $setting);
    $this->set('categories', $categories);
    //$this->help = 'schedule_configs_form';
    $this->render('form');
  }


  /**
  * [ADMIN] 編集画面
  *
  * @return void
  */
  public function admin_edit($id = null) {
    /* 除外処理 */
    if (empty($id)) {
      $this->setMessage('無効なIDです。', true);
      $this->redirect(array('action' => 'index'));
    }

    /* 設定情報の取得 */
    $setting = $this->ScheduleSetting->find('first');

    //インストール直後など、値が空の場合に備えデフォルト値を付けておく。
    if (empty($setting)) {
      $setting['ScheduleSetting']['editor_type'] = 'ckeditor';
      $setting['ScheduleSetting']['image01'] = 0;
      $setting['ScheduleSetting']['image02'] = 0;
      $setting['ScheduleSetting']['image03'] = 0;
      $setting['ScheduleSetting']['image04'] = 0;
      $setting['ScheduleSetting']['map'] = 0;
    }

    //カテゴリ情報の取得
    $categories = $this->ScheduleCategory->find('list', array('conditions' => array(
      'ScheduleCategory.publish' => 1
    )));

    //更新ボタンを押下したかどうか
    if ($this->request->data) {
      //画像の最大値を確認する
      if (!empty($setting['ScheduleSetting']['max_width'])) {
        $maxWidth = $setting['ScheduleSetting']['max_width'];
      } else {
        $maxWidth = null;
      }
      if (!empty($setting['ScheduleSetting']['max_height'])) {
        $maxHeight = $setting['ScheduleSetting']['max_height'];
      } else {
        $maxHeight = null;
      }
      //画像保存処理実行
      $insertData = $this->Schedule->savePostImg($this->request->data, $maxWidth, $maxHeight);
      //保存
      if ($this->Schedule->save($insertData)) {
        $this->setMessage('スケジュール'. h($this->request->data['Schedule']['title']) .'」を編集しました。', false, true);
        $this->redirect(array('action' => 'index'));
      } else {
        $this->setMessage('入力エラーです。内容を修正してください。', true);
      }

    //初期画面
    } else {
      //既存情報の取得
      $this->request->data = $this->Schedule->findById($id);
      $this->set('schedule', $this->request->data);
      //画像保存パス
      $this->set('imgPath', '/img/schedule_plugin/');
    }

    //エディタ設定
    $editorOptions = array('editorDisableDraft' => true);
    if (!empty($this->siteConfigs['editor_styles'])) {
      App::uses('CKEditorStyleParser', 'Vendor');
      $CKEditorStyleParser = new CKEditorStyleParser();
      $editorStyles = array('default' => $CKEditorStyleParser->parse($this->siteConfigs['editor_styles']));
      $editorOptions = array_merge($editorOptions, array(
        'editorStylesSet' => 'simple',
        'editorStyles' => $editorStyles
      ));
    }
    $this->set('editorOptions', $editorOptions);

    /* 表示設定 */
    $this->pageTitle = 'スケジュール 編集';
    $this->set('setting', $setting);
    $this->set('categories', $categories);
    //$this->help = 'schedule_configs_form';
    $this->render('form');
  }


  /**
  * [ADMIN] 削除
  *
  * @param int $id スケジュールID
  * @return void
  */
  public function admin_delete($id = null) {
    /* 除外処理 */
    if (empty($id)) {
      $this->setMessage('無効なIDです。', true);
      $this->redirect(array('action' => 'index'));
    }
    //情報の取得
    $schedule = $this->Schedule->findById($id);
    //削除実行
    if ($this->Schedule->delete($id)) {
      //画像も削除する。
      for ($i = 1; $i <= 4; $i++) {
        if (!empty($schedule['Schedule']['image_'.$i])) {
          $this->Schedule->deleteSaveImage($schedule['Schedule']['image_'.$i]);
        }
      }
    } else {
      $this->setMessage('入力エラーです。削除に失敗しました。', true);
    }
    /* 表示設定 */
    $this->redirect(array('action' => 'index'));
  }


  /**
  * [ADMIN] 公開設定切替
  *
  * @return void
  */
  public function admin_publish($id){
    /* 除外処理 */
    if (empty($id)){
      $this->setMessage('無効なIDです。', true);
      $this->redirect(array('action' => 'index'));
    }
    //現在の情報を取得する。
    $data = $this->Schedule->find('first', array('conditions' => array(
      'Schedule.id' => $id
    )));
    //非公開設定
    switch($data['Schedule']['publish']){
      case 1:
      $data['Schedule']['publish'] = 0;
      break;
      case 0:
      $data['Schedule']['publish'] = 1;
      break;
    }

    //更新処理実行
    $this->Schedule->id = $id;
    if ($this->Schedule->save($data)) {
      $this->setMessage('スケジュール'. h($data['Schedule']['title']). '」の公開設定を切替ました。', false, true);
      $this->redirect(array('action' => 'admin_index'));
    } else {
      //$this->setMessage('入力エラーです。内容を修正してください。', true);
    }
  }


  /**
  * 並び替えを更新する [AJAX]
  *
  * @access public
  * @return boolean
  */
  public function admin_ajax_update_sort() {
  	if ($this->request->data) {
  		$this->setViewConditions('ScheduleConfigs', array('action' => 'admin_index'));
      $conditions = array();
  		if ($this->Schedule->changeSort($this->request->data['Sort']['id'], $this->request->data['Sort']['offset'], $conditions)) {
  			clearViewCache();
  			clearDataCache();
  			echo true;
  		} else {
  			$this->ajaxError(500, '一度リロードしてから再実行してみてください。');
  		}
  	} else {
  		$this->ajaxError(500, '無効な処理です。');
  	}
  	exit();
  }
}
