<?php
/*
* スケジュールプラグイン
* 休日管理コントローラー
*
* PHP 5.4.x
*
* @copyright    Copyright (c) hiniarata co.ltd
* @link         https://hiniarata.jp
* @package      Schedule Plugin Project
* @since        ver.0.9.1
*/

/**
* スケジュール　休日管理コントローラー
*
* @package baser.plugins.schedule
*/
class ScheduleHolidayConfigsController extends ScheduleAppController {

  /**
  * クラス名
  *
  * @var string
  * @access public
  */
  public $name = 'ScheduleHolidayConfigs';

  /**
  * コンポーネント
  *
  * @var array
  * @access public
  */
  public $components = array('BcAuth', 'Cookie', 'BcAuthConfigure');

  /**
  * モデル
  *
  * @var array
  * @access public
  */
  public $uses = array('Schedule.ScheduleHoliday', 'Schedule.ScheduleHolidaySetting');

  /**
  * ぱんくずナビ
  *
  * @var string
  * @access public
  */
  public $crumbs = array(
  array('name' => 'プラグイン管理', 'url' => array('plugin' => '', 'controller' => 'plugins', 'action' => 'index')),
  array('name' => 'スケジュール管理', 'url' => array('controller' => 'schedule_configs', 'action' => 'index'))
  );

  /**
  * サブメニューエレメント
  *
  * @var array
  * @access public
  */
  public $subMenuElements = array('schedule');

  /**
  * beforeFilter
  *
  * @return	void
  * @access 	public
  */
  public function beforeFilter() {
    parent::beforeFilter();
  }


  /**
  * [ADMIN] 設定済月を一覧表示
  *
  * @return void
  */
  public function admin_index() {
    // 画面表示情報設定
    $default = array(
      'named' => array(
        'num' => $this->siteConfigs['admin_list_num']
      )
    );
    $this->setViewConditions('ScheduleHoliday', array('default' => $default));
    //データの取得
    $conditions = array();
    $this->paginate = array('conditions' => $conditions,
      'order' => 'ScheduleHoliday.id DESC',
    'limit' => $this->passedArgs['num'],
    );
    /* 表示設定 */
    $this->set('schedule', $this->paginate('ScheduleHoliday'));
    $this->pageTitle = '休日 設定済の月 一覧';
    $this->help = 'schedule_holidays_index';
  }


  /**
  * [ADMIN] STEP.1  設定月の選択画面
  *
  * @return void
  */
  public function admin_add() {
    //本年取得
    $thisYear = date("Y");
    $thisMonth = date("m");
    //未来は五年先までくらいでOK
    $years = array("$thisYear" => $thisYear);
    for($i=1; $i<=5; $i++){
      $y = (int)$thisYear + $i;
      $years["$y"] = $y;
    }
    $months = array(
      '1' => 1, '2' => 2,'3' => 3,'4' => 4,'5' => 5,'6' => 6,
      '7' => 7,'8' => 8,'9' => 9,'10' => 10,'11' => 11,'12' => 12);

    //間近のデータを取得する。
    $lastSetting = $this->ScheduleHoliday->find('first', array(
      'order' => array(
        'ScheduleHoliday.first_day' => 'DESC'
    )));
    //基本設定を確認する。
    $baseSetting = $this->ScheduleHolidaySetting->find('first');

    //もしも、設定済みが無ければ（初期）、本年本月をselectedにする。
    if (empty($lastSetting)) {
      $selectYear = $thisYear;
      $selectMonth = $thisMonth;
    } else {
      $settingYear = date("Y", strtotime($lastSetting['ScheduleHoliday']['first_day']));
      $settingMonth = date("m", strtotime($lastSetting['ScheduleHoliday']['first_day']));
      //もしも12月なら翌年１月を設定する。
      if($settingMonth == 12){
        $selectMonth = 1;
        $selectYear = (int)$settingYear + 1;
      } else {
        $selectMonth = (int)$settingMonth +1;
        $selectYear = $settingYear;
      }
    }
    //まずはステップ１
    //月の選択だけやって、POST後に既存の設定がないか確認する？
    $this->set('selectYear', $selectYear);
    $this->set('selectMonth', $selectMonth);
    $this->set('lastSetting', $lastSetting);
    $this->set('baseSetting', $baseSetting);
    $this->set('years', $years);
    $this->set('months', $months);
    $this->pageTitle = '休日 月の選択';
  }


  /**
  * [ADMIN] STEP.2  カレンダーの表示
  *
  * @return void
  */
  public function admin_step2(){
    //除外処理
    if (empty($this->request->data)) {
      $this->setMessage('無効な操作です', true);
      $this->redirect(array('action' => 'index'));
    }

    //生成月
    $thisFirstDay = date("Y-m-d", strtotime($this->request->data['ScheduleHoliday']['year'].'-'.sprintf('%02d', $this->request->data['ScheduleHoliday']['month']).'-01'));
    //この月があるかどうかを確認。
    $count = $this->ScheduleHoliday->find('count', array('conditions'=>array(
      'ScheduleHoliday.first_day' => $thisFirstDay
    )));
    if ($count > 0) {
      $this->setMessage('指定された年月は、既にカレンダーが設定されているようです。', true);
      $this->redirect(array('action' => 'add'));
    }

    //基本設定の情報を取得する。
    $setting = $this->ScheduleHolidaySetting->findById(1);
    $weekHoliday = array();
    //アンシリアライズ
    //1週目
    if (!empty($setting['ScheduleHolidaySetting']['week1'])) {
      $setting['ScheduleHolidaySetting']['week1'] = unserialize($setting['ScheduleHolidaySetting']['week1']);
      //曜日番号をキーとする配列を生成する。
      foreach($setting['ScheduleHolidaySetting']['week1'] as $data){
        $weekHoliday['week1'][$data] = 'on';
      }
    }
    //2周目
    if (!empty($setting['ScheduleHolidaySetting']['week2'])) {
      $setting['ScheduleHolidaySetting']['week2'] = unserialize($setting['ScheduleHolidaySetting']['week2']);
      foreach($setting['ScheduleHolidaySetting']['week2'] as $data){
        $weekHoliday['week2'][$data] = 'on';
      }
    }
    //3週目
    if (!empty($setting['ScheduleHolidaySetting']['week3'])) {
      $setting['ScheduleHolidaySetting']['week3'] = unserialize($setting['ScheduleHolidaySetting']['week3']);
      foreach($setting['ScheduleHolidaySetting']['week3'] as $data){
        $weekHoliday['week3'][$data] = 'on';
      }
    }
    //4周目
    if (!empty($setting['ScheduleHolidaySetting']['week4'])) {
      $setting['ScheduleHolidaySetting']['week4'] = unserialize($setting['ScheduleHolidaySetting']['week4']);
      foreach($setting['ScheduleHolidaySetting']['week4'] as $data){
        $weekHoliday['week4'][$data] = 'on';
      }
    }
    //5周目
    if (!empty($setting['ScheduleHolidaySetting']['week5'])) {
      $setting['ScheduleHolidaySetting']['week5'] = unserialize($setting['ScheduleHolidaySetting']['week5']);
      foreach($setting['ScheduleHolidaySetting']['week5'] as $data){
        $weekHoliday['week5'][$data] = 'on';
      }
    }
    //6周目
    if (!empty($setting['ScheduleHolidaySetting']['week6'])) {
      $setting['ScheduleHolidaySetting']['week6'] = unserialize($setting['ScheduleHolidaySetting']['week6']);
      foreach($setting['ScheduleHolidaySetting']['week6'] as $data){
        $weekHoliday['week6'][$data] = 'on';
      }
    }

    //表示
    $this->set('weekHoliday', $weekHoliday);
    $this->set('year', $this->request->data['ScheduleHoliday']['year']);
    $this->set('month', $this->request->data['ScheduleHoliday']['month']);
    $this->pageTitle = '休日カレンダーの作成';
    $this->render('form');
  }


  /**
  * [ADMIN] STEP.3  データの取得と登録
  *
  * @return void
  */
  public function admin_step3(){
    //除外処理
    if (empty($this->request->data)) {
      $this->setMessage('無効な操作です', true);
      $this->redirect(array('action' => 'index'));
    }

    //配列で取得する。
    if (!empty($this->request->data['ScheduleHoliday']['holiday'])) {
      $this->request->data['ScheduleHoliday']['holidays'] = serialize($this->request->data['ScheduleHoliday']['holiday']);
    }

    //その月の１日をセットする。
    $this->request->data['ScheduleHoliday']['first_day'] = date("Y-m-d", strtotime($this->request->data['ScheduleHoliday']['year'].'-'.sprintf('%02d', $this->request->data['ScheduleHoliday']['month']).'-01'));

    //保存処理
    if ($this->ScheduleHoliday->save($this->request->data)) {
      $this->setMessage($this->request->data['ScheduleHoliday']['year'].'年'.$this->request->data['ScheduleHoliday']['month'].'月の休日カレンダーを生成しました。', false, true);
      $this->redirect(array('action' => 'admin_index'));
    } else {
      $this->setMessage('保存処理に失敗しました。', true);
    }
  }


  /**
  * [ADMIN] 設定済月を編集する
  *
  * @return void
  */
  public function admin_edit($id =null ) {
    //除外処理
    if (empty($id)) {
      $this->setMessage('無効なIDです', true);
      $this->redirect(array('action' => 'index'));
    }

    //編集ボタンの押下後
    if (!empty($this->request->data)) {
      //配列で取得する。
      if (!empty($this->request->data['ScheduleHoliday']['holiday'])) {
        $this->request->data['ScheduleHoliday']['holidays'] = serialize($this->request->data['ScheduleHoliday']['holiday']);
      }

      //その月の１日をセットする。
      $this->request->data['ScheduleHoliday']['first_day'] = date("Y-m-d", strtotime($this->request->data['ScheduleHoliday']['year'].'-'.sprintf('%02d', $this->request->data['ScheduleHoliday']['month']).'-01'));

      //保存処理
      if ($this->ScheduleHoliday->save($this->request->data)) {
        $this->setMessage($this->request->data['ScheduleHoliday']['year'].'年'.$this->request->data['ScheduleHoliday']['month'].'月の休日カレンダーを編集しました。', false, true);
        $this->redirect(array('action' => 'admin_index'));
      } else {
        $this->setMessage('保存処理に失敗しました。', true);
      }
    }

    //データの取得
    $holidayData = $this->ScheduleHoliday->find('first', array('conditions' => array(
      'ScheduleHoliday.id' => $id
    )));
    //除外処理
    if (empty($holidayData)) {
      $this->setMessage('無効なIDです', true);
      $this->redirect(array('action' => 'index'));
    }
    //日付とチェックの有無がわかるように配列を整理する。
    $checkHoliday = array();
    //その月に休日が１日でもセットされていたら、その日をキーにした配列を組む。
    if (!empty($holidayData['ScheduleHoliday']['holidays'])) {
      //アンシリアライズ
      $holidays = unserialize($holidayData['ScheduleHoliday']['holidays']);
      foreach ($holidays as $data) {
        $checkHoliday[$data] = 'on';
      }
    }
    //表示
    $this->set('checkHoliday', $checkHoliday);
    $this->set('id', $id);
    $this->set('year', date("Y", strtotime($holidayData['ScheduleHoliday']['first_day'])));
    $this->set('month', date("m", strtotime($holidayData['ScheduleHoliday']['first_day'])));
    $this->pageTitle = '休日カレンダーの編集';
    $this->render('form');
  }


  /**
  * [ADMIN] 設定済月を編集する
  *
  * @return void
  */
  public function admin_setting($id = null) {
    //ボタン押下後
    if (!empty($this->request->data)) {
      //シリアライズ
      //1週目
      if (!empty($this->request->data['ScheduleHolidaySetting']['week1'])) {
        $this->request->data['ScheduleHolidaySetting']['week1'] = serialize($this->request->data['ScheduleHolidaySetting']['week1']);
      }
      //2周目
      if (!empty($this->request->data['ScheduleHolidaySetting']['week2'])) {
        $this->request->data['ScheduleHolidaySetting']['week2'] = serialize($this->request->data['ScheduleHolidaySetting']['week2']);
      }
      //3週目
      if (!empty($this->request->data['ScheduleHolidaySetting']['week3'])) {
        $this->request->data['ScheduleHolidaySetting']['week3'] = serialize($this->request->data['ScheduleHolidaySetting']['week3']);
      }
      //4周目
      if (!empty($this->request->data['ScheduleHolidaySetting']['week4'])) {
        $this->request->data['ScheduleHolidaySetting']['week4'] = serialize($this->request->data['ScheduleHolidaySetting']['week4']);
      }
      //5周目
      if (!empty($this->request->data['ScheduleHolidaySetting']['week5'])) {
        $this->request->data['ScheduleHolidaySetting']['week5'] = serialize($this->request->data['ScheduleHolidaySetting']['week5']);
      }
      //6周目
      if (!empty($this->request->data['ScheduleHolidaySetting']['week6'])) {
        $this->request->data['ScheduleHolidaySetting']['week6'] = serialize($this->request->data['ScheduleHolidaySetting']['week6']);
      }

      //id:1
      $this->request->data['ScheduleHolidaySetting']['id'] = 1;

      //保存処理
      if ($this->ScheduleHolidaySetting->save($this->request->data)) {
        $this->setMessage('基本設定を保存しました。', false, true);
        $this->redirect(array('action' => 'admin_index'));
      } else {
        $this->setMessage('保存処理に失敗しました。', true);
      }
    }

    //既存設定の取得
    $setting = $this->ScheduleHolidaySetting->findById(1);
    //アンシリアライズ
    if (!empty($setting['ScheduleHolidaySetting']['week1'])) {
      $setting['ScheduleHolidaySetting']['week1'] = unserialize($setting['ScheduleHolidaySetting']['week1']);
    }
    if (!empty($setting['ScheduleHolidaySetting']['week2'])) {
      $setting['ScheduleHolidaySetting']['week2'] = unserialize($setting['ScheduleHolidaySetting']['week2']);
    }
    if (!empty($setting['ScheduleHolidaySetting']['week3'])) {
      $setting['ScheduleHolidaySetting']['week3'] = unserialize($setting['ScheduleHolidaySetting']['week3']);
    }
    if (!empty($setting['ScheduleHolidaySetting']['week4'])) {
      $setting['ScheduleHolidaySetting']['week4'] = unserialize($setting['ScheduleHolidaySetting']['week4']);
    }
    if (!empty($setting['ScheduleHolidaySetting']['week5'])) {
      $setting['ScheduleHolidaySetting']['week5'] = unserialize($setting['ScheduleHolidaySetting']['week5']);
    }
    if (!empty($setting['ScheduleHolidaySetting']['week6'])) {
      $setting['ScheduleHolidaySetting']['week6'] = unserialize($setting['ScheduleHolidaySetting']['week6']);
    }

    //表示設定
    $this->request->data = $setting;
    $this->pageTitle = '休日カレンダー 基本設定';
  }


  /**
  * [ADMIN] 休日設定 削除
  *
  * @param int $id スケジュールID
  * @return void
  */
  public function admin_delete($id = null) {
    /* 除外処理 */
    if (empty($id)) {
      $this->setMessage('無効なIDです。', true);
      $this->redirect(array('action' => 'index'));
    }

    //情報の取得
    $schedule = $this->ScheduleHoliday->findById($id);
    $thisMonth = date("Y年m月", strtotime($schedule['ScheduleHoliday']['first_day']));

    if ($this->ScheduleHoliday->delete($id)) {
      $this->setMessage($thisMonth.'の休日カレンダー設定を初期化しました。', false, true);
    } else {
      $this->setMessage('入力エラーです。削除に失敗しました。', true);
    }

    /* 表示設定 */
    $this->redirect(array('action' => 'index'));
  }
}
