<?php
/*
* スケジュールプラグイン
* 管理設定コントローラー
*
* PHP 5.4.x
*
* @copyright    Copyright (c) hiniarata co.ltd
* @link         https://hiniarata.jp
* @package      Schedule Plugin Project
* @since        ver.0.9.0
*/

/**
* スケジュール 管理設定コントローラー
*
* @package baser.plugins.schedule
*/
class ScheduleSettingsController extends ScheduleAppController {

  /**
  * クラス名
  *
  * @var string
  * @access public
  */
  public $name = 'ScheduleSettings';

  /**
  * コンポーネント
  *
  * @var array
  * @access public
  */
  public $components = array('BcAuth', 'Cookie', 'BcAuthConfigure');

  /**
  * モデル
  *
  * @var array
  * @access public
  */
  public $uses = array('Schedule.ScheduleSetting');

  /**
  * ぱんくずナビ
  *
  * @var string
  * @access public
  */
  public $crumbs = array(
  array('name' => 'プラグイン管理', 'url' => array('plugin' => '', 'controller' => 'plugins', 'action' => 'index')),
  array('name' => 'スケジュール設定', 'url' => array('controller' => 'schedule_settings', 'action' => 'index'))
  );

  /**
  * サブメニューエレメント
  *
  * @var array
  * @access public
  */
  public $subMenuElements = array('schedule');

  /**
  * beforeFilter
  *
  * @return void
  * @access   public
  */
  public function beforeFilter() {
    parent::beforeFilter();
  }

  /**
  * [ADMIN] 現在の設定状況
  *
  * @return void
  */
  public function admin_index() {
    //データの取得
    $setting = $this->ScheduleSetting->find('first');
    /* 表示設定 */
    $this->set('setting', $setting);
    $this->pageTitle = 'スケジュール 現在の設定';
  }

  /**
  * [ADMIN] 設定の変更
  *
  * @return void
  */
  public function admin_edit($id = null) {
    //更新ボタンの押下確認
    if (!empty($this->request->data)) {
      if (!empty($id)) {
        //データの存在確認
        $settingData = $this->ScheduleSetting->findById($id);
        if (empty($settingData)) {
          $this->setMessage('無効なIDです。', true);
          $this->redirect(array('action' => 'index'));
        //データが確認できた場合、上書き保存のためのidセット
        } else {
          $this->ScheduleSetting->id = $id;
        }
      }
      if ($this->ScheduleSetting->save($this->request->data)) {
        $this->setMessage('スケジュールプラグインの設定を変更しました。', false, true);
        $this->redirect(array('action' => 'index'));
      } else {
        $this->setMessage('入力エラーです。内容を修正してください。', true);
      }
    }

    //現在の設定取得
    $setting = $this->ScheduleSetting->find('first');
    //画像の幅
    if (!empty($setting['ScheduleSetting']['max_width'])) {
      $maxWidth = $setting['ScheduleSetting']['max_width'];
    } else {
      $maxWidth = 500; //デフォルト値とする
    }
    //画像の高さ
    if (!empty($setting['ScheduleSetting']['max_height'])) {
      $maxHeight = $setting['ScheduleSetting']['max_height'];
    } else {
      $maxHeight = 500; //デフォルト値とする
    }

    /* 表示設定 */
    $this->set('maxWidth', $maxWidth);
    $this->set('maxHeight', $maxHeight);
    $this->request->data = $setting;
    $this->set('setting', $setting);
    $this->pageTitle = 'スケジュール 設定変更';
    $this->help = 'schedule_settings';
    $this->render('form');
  }

  /**
  * [ADMIN] 設定の初期化
  *
  * @return void
  */
  public function admin_delete($id = null) {
    if (empty($id)) {
      $this->setMessage('無効なIDです。', true);
      $this->redirect(array('action' => 'index'));
    }
    //削除処理
    if ($this->ScheduleSetting->delete($id)) {
      $this->setMessage('スケジュールプラグインの設定を初期化しました。', false, true);
    } else {
      $this->setMessage('エラーです。初期化に失敗しました。', true);
    }
    /* 表示設定 */
    $this->redirect(array('action' => 'index'));
  }
}
