<?php
/*
* スケジュールプラグイン
* フロント側 コントローラー
*
* PHP 5.4.x
*
* @copyright    Copyright (c) hiniarata co.ltd
* @link         https://hiniarata.jp
* @package      Schedule Plugin Project
* @since        ver.0.9.0
*/

/**
*  フロント側 コントローラー
*
* @package    baser.plugins.schedule
*/
class SchedulesController extends ScheduleAppController {

  /**
  * クラス名
  *
  * @var string
  * @access public
  */
  public $name = 'Schedules';

  /**
  * コンポーネント
  *
  * @var array
  * @access public
  */
  public $components = array();

  /**
  * モデル
  *
  * @var array
  * @access public
  */
  public $uses = array('Schedule.Schedule', 'Schedule.ScheduleCategory');

  /**
  * beforeFilter
  *
  * @return	void
  * @access 	public
  */
  public function beforeFilter() {
    parent::beforeFilter();
  }

  /**
  *  一覧表示
  *
  * @param int $categoryId カテゴリID
  * @return void
  */
  public function index($categoryId = null){
    //1ページの表示件数
    $limit = 10;
    //検索条件
    $conditions = array();
    $conditions[] = array('Schedule.publish' => 1); //公開中
    $conditions[] = array('OR' => array(
          array('Schedule.schedule_category_id' => null),
          array('ScheduleCategory.publish' => 1))
        ); //利用中

    //カテゴリ指定があった場合
    if (!empty($categoryId)) {
      $conditions[] = array('Schedule.schedule_category_id' => $categoryId);
      //コンテンツテンプレートの設定を読み込む
      $categoryData = $this->ScheduleCategory->findById($categoryId);
      if (!empty($categoryData['ScheduleCategory']['template'])) {
        $template = $categoryData['ScheduleCategory']['template'] . DS . 'index';
      }
    }
    //ページネート
    $this->paginate = array('conditions' => $conditions,
      //'order' => 'Schedule.id DESC',
      'order' => 'Schedule.sort ASC',
      'limit' => $limit
    );

    //ここまでの所でコンテンツテンプレートが設定なし
    //初期テンプレートを使用する
    if (empty($template)) {
      $template = 'default' . DS . 'index';
    }

    /* 表示設定 */
    $this->set('schedules', $this->paginate('Schedule'));
    $this->pageTitle = 'スケジュール 一覧';
    $this->render($template);
  }

/**
 * [MOBILE] 一覧表示
 *
 * @return void
 * @access public
 */
  public function mobile_index($categoryId = null) {
    $this->setAction('index', $categoryId);
  }

/**
 * [SMARTPHONE] 一覧表示
 *
 * @return void
 * @access public
 */
  public function smartphone_index($categoryId = null) {
    $this->setAction('index', $categoryId);
  }

  /**
  *  個別表示
  *
  * @param int $scheduleId スケジュールID
  * @return void
  */
  public function view($scheduleId = null){
    /* 除外処理 */
    if (empty($scheduleId)) {
      $this->redirect(array('action' => 'index'));
    }
    //データの取得
    $scheduleData = $this->Schedule->find('first', array('conditions' => array(
        'Schedule.id' => $scheduleId,
        'Schedule.publish' => 1,
        'AND' => array('OR' => array(
          array('ScheduleCategory.publish' => 1),
          array('Schedule.schedule_category_id' => null)
        ))
    )));
    if (empty($scheduleData)) {
      $this->redirect(array('action' => 'index'));
    }

    //カテゴリ指定があった場合はテンプレートを確認する
    if (!empty($scheduleData['ScheduleCategory']['template'])) {
      $template = $scheduleData['ScheduleCategory']['template'] . DS . 'view';
    }
    //カテゴリに入っていない、テンプレートの指定がないなどの場合
    if (empty($template)) {
      $template = 'default' . DS . 'view';
    }

    /* 表示設定 */
    $this->set('schedule', $scheduleData);
    $this->pageTitle = $scheduleData['Schedule']['title'];
    $this->render($template);
  }

/**
 * [MOBILE] 個別表示
 *
 * @return void
 * @access public
 */
  public function mobile_view($id = null) {
    $this->setAction('view', $id);
  }

/**
 * [SMARTPHONE] 個別表示
 *
 * @return void
 * @access public
 */
  public function smartphone_view($id = null) {
    $this->setAction('view', $id);
  }


  /**
  *  特定の日付の一覧
  *
  * @param int $year 年
  * @param int $month 月
  * @param int $day 日
  * @return void
  */
  public function day($year = null, $month = null, $day = null){
    /* 除外処理 */
    if (empty($year)) {
      $this->redirect(array('action' => 'index'));
    }
    if (empty($month)) {
      $this->redirect(array('action' => 'index'));
    }
    if (empty($day)) {
      $this->redirect(array('action' => 'index'));
    }

    $start = date("Y-m-d H:i", strtotime($year .'-'. $month .'-'. $day . ' 23:59:59'));
    $end = date("Y-m-d H:i", strtotime($year .'-'. $month .'-'. $day . ' 00:00:00'));

    //データの取得
    //検索条件
    $conditions = array(
      'Schedule.publish' => 1,
      'Schedule.begin_date <=' => $start,
      'Schedule.end_date >=' =>  $end
    );

    //1ページの表示件数
    $limit = 10;
    //ページネート
    $this->paginate = array('conditions' => $conditions,
      'order' => 'Schedule.sort ASC',
      'limit' => $limit
    );

    //いろいろなカテゴリの記事があるので、ここはdefaultで。
    //カテゴリに入っていない、テンプレートの指定がないなどの場合
    if (empty($template)) {
      $template = 'default' . DS . 'day';
    }

    /* 表示設定 */
    $this->set('schedules', $this->paginate('Schedule'));
    $this->set('year', $year);
    $this->set('month', $month);
    $this->set('day', $day);
    $this->pageTitle = $year .'年'. $month .'月'. $day. '日のスケジュール';
    $this->render($template);
  }


  /**
  *  新着一覧
  *  RequestAction用
  *
  * @param $num 取得件数
  * @return void
  */
  public function schedules($num = 5){
    //検索条件
    $conditions = array();
    if (!empty($this->params['named']['category'])) {
      $conditions[] = array('Schedule.schedule_category_id' => $this->params['named']['category']);
      //カテゴリデータの取得
      $categoryData = $this->ScheduleCategory->findById($this->params['named']['category']);
      if(!empty($categoryData['ScheduleCategory']['template'])){
        $contentTemplate = $categoryData['ScheduleCategory']['template'];
      }
    }

    //テンプレートの指定
    if (!empty($this->params['named']['template'])) {
      $template = $this->request->params['named']['template'];
    } else {
      $template = 'lists';
    }
    unset($this->request->params['named']['template']);

    //レイアウトファイルの無効化
    $this->layout = null;

    //データの取得
    $conditions[] = array('Schedule.publish' => 1);
    $schedules = $this->Schedule->find('all', array(
      'conditions' => $conditions,
      'limit' => $num
    ));

    $this->set('schedules', $schedules);
    //レンダリング（カテゴリにコンテンツテンプレートが設定されている場合はそれを指定する）
    if (!empty($contentTemplate)) {
      $this->render($contentTemplate . DS . $template);
    } else {
      $this->render('default' . DS . $template);
    }
  }


/**
 * [MOBILE] 新着一覧
 *
 * requestAction用
 *
 * @param int $num 取得件数
 * @access public
 */
	public function mobile_schedules($limit = 5) {
		$this->setAction('schedules', $limit);
	}


/**
 * [SMARTPHONE] 新着一覧
 *
 * requestAction用
 *
 * @param int $num 取得件数
 * @access public
 */
	public function smartphone_schedules($limit = 5) {
		$this->setAction('schedules', $limit);
	}


}
