<?php
/*
* スケジュールプラグイン
* 情報管理モデル
*
* PHP 5.4.x
*
* @copyright    Copyright (c) hiniarata co.ltd
* @link         https://hiniarata.jp
* @package      Schedule Plugin Project
* @since        ver.0.9.0
*/

/**
* Include files
*/
App::uses('ScheduleApp', 'Schedule.Model');
App::uses('Imageresizer', 'Vendor'); //標準搭載の画像処理クラス

/**
* 登録情報モデル
*
* @package baser.plugins.schedule
*/
class Schedule extends ScheduleApp {
  /**
  * クラス名
  *
  * @var string
  * @access public
  */
  public $name = 'Schedule';

  /**
  * プラグイン名
  *
  * @var string
  * @access public
  */
  public $plugin = 'Schedule';

  /**
  * DB接続時の設定
  *
  * @var string
  * @access public
  */
  public $useDbConfig = 'plugin';

  /**
  * アソシエーション
  *
  * @var string
  * @access public
  */
  public $belongsTo = array(
    'ScheduleCategory' => array(
      'className'    => 'Schedule.ScheduleCategory',
      'foreignKey'   => 'schedule_category_id'
    )
  );

  /**
  * 画像保存パス
  *
  * @var string
  * @access public
  */
  public $imageDir = 'schedule_plugin';

  /**
  * 画像のデフォルト幅（px）
  *
  * @var string
  * @access public
  */
  public $widthSize = 500;

  /**
  * 画像のデフォルト高さ（px）
  *
  * @var string
  * @access public
  */
  public $heightSize = 500;

  /**
  * 画像の枚数
  *
  * @var string
  * @access public
  */
  public $imageCount = 4;

  /**
  * validate
  *
  * @var array
  */
  public $validate = array(
    'title' => array(
      'notEmpty' => array(
        'rule' => array('notEmpty'),
        'message' => 'タイトルを入力してください。'
    )),
    'text' => array(
      'notEmpty' => array(
        'rule' => array('notEmpty'),
        'message' => '本文を入力してください。'
    )),
    'begin_date' => array(
      'notEmpty' => array(
        'rule' => array('notEmpty'),
        'message' => '開始日時を入力してください。'
    )),
    'end_date' => array(
      'notEmpty' => array(
        'rule' => array('notEmpty'),
        'message' => '終了日時を入力してください。'
    ))
  );


  /**
  * 画像を保存する。
  *
  * @params array $data
  * @params init $width
  * @params init $height
  * @return bool
  * @access public
  */
  public function savePostImg($data, $width = null, $height = null){

    //現在のデータを取得する。（画像に変更がなかった場合の処理）
    if (!empty($data['Schedule']['id'])) {
      $scheduleData = $this->find('first', array('conditions' => array(
        'Schedule.id' => $data['Schedule']['id']
      )));
    }

    //画像の幅指定
    if (!empty($width)) {
      $limitWidth = $width;
    } else {
      $limitWidth = $this->widthSize; //デフォルト
    }
    if (!empty($height)) {
      $limitHeight = $height;
    } else {
      $limitHeight = $this->heightSize; //デフォルト
    }

    //ファイル送信の確認と保存処理
    for ($i = 1; $i <= $this->imageCount; $i++) {
      if (empty($data['Schedule']['del_image_'.$i]) || $data['Schedule']['del_image_'.$i] == 0) { //削除指定がない場合にのみアップ作業
        if (!empty($data['Schedule']['image_'.$i]['tmp_name'])) {
          //現在のファイル名と、そこから拡張子を確認する。
          $img = $this->_copyTmpImage($data['Schedule']['image_'.$i], 'schedule_'.$i.'_' , $limitWidth, $limitHeight);
          $data['Schedule']['image_'.$i] = $img;

        //ファイル送信なしの場合
        } else {
          //旧画像名をそのままセットする。
          if (!empty($scheduleData['Schedule']['image_'.$i])) {
            $data['Schedule']['image_'.$i] = $scheduleData['Schedule']['image_'.$i];
          } else {
            $data['Schedule']['image_'.$i] = "";
          }
        }

      //既存のファイルの削除指示がある場合。
      } else {
        $this->_deletePostImage('image_'.$i, $data);
        $data['Schedule']['image_'.$i] = ''; //フィールドも空をセット
      }
    }
    return $data;
  }

  /**
  * 画像を削除する。
  *
  * @params string $imgKey
  * @params array $data
  * @return bool
  * @access public
  */
  public function deleteSaveImage($imageName){
    //削除処理
    if (!empty($imageName)) {
      $delFileName = WWW_ROOT . 'img' . DS . $this->imageDir . DS . $imageName;
      if (file_exists($delFileName)) {
        @unlink($delFileName);
        return true;
      }else{
        return false;
      }
    }
  }

  /**
  * 画像データだけの配列を返す
  *
  * @params int $categoryId
  * @return array
  * @access public
  */
  public function getImageGallary($categoryId = null){
    //画像ギャラリー格納用
    $gallary = array();
    //検索条件
    $conditions = array();
    //カテゴリ有無
    if (!empty($categoryId)) {
      $conditions[] = array('Schedule.schedule_category_id' => $categoryId);
    }
    //必須条件
    $conditions[] = array(
                      'Schedule.publish' => 1,
                      'AND' => array('OR' => array(
                        array('ScheduleCategory.publish' => 1),
                        array('Schedule.schedule_category_id' => null)
                    )));
    //データの取得
    $scheduleData = $this->find('all', array('conditions' => $conditions));
    //取得した画像を整理する。
    foreach ($scheduleData as $data) {
      $images = array(); //一時的に画像データとidを保存する。
      if (!empty($data['Schedule']['image_1'])) {
        $images['image'] = $data['Schedule']['image_1'];
        $images['id'] = $data['Schedule']['id'];
        $gallary[] = $images;
      }
      if (!empty($data['Schedule']['image_2'])) {
        $images['image'] = $data['Schedule']['image_2'];
        $images['id'] = $data['Schedule']['id'];
        $gallary[] = $images;
      }
      if (!empty($data['Schedule']['image_3'])) {
        $images['image'] = $data['Schedule']['image_3'];
        $images['id'] = $data['Schedule']['id'];
        $gallary[] = $images;
      }
      if (!empty($data['Schedule']['image_4'])) {
        $images['image'] = $data['Schedule']['image_4'];
        $images['id'] = $data['Schedule']['id'];
        $gallary[] = $images;
      }
    }
    //結果を返す
    return $gallary; //連想配列で[image][id]を持つ。
  }

  /**
  * 一時ファイルから公開フォルダ上へコピーして
  * 新しいファイル名を返す。
  *
  * @params array $data
  * @params string $prefix
  * @params int $width
  * @params int $height
  * @return string
  * @access protected
  */
  protected function _copyTmpImage($data, $prefix, $width, $height) {
    //コピー元
    $fileName = $data['name'];
    $ext = pathinfo($fileName, PATHINFO_EXTENSION);
    //新しいファイル名を生成
    $fileNamePrefix = $prefix;
    $fileNameNo = date("YmdHis");
    $newFileName =  $fileNamePrefix.$fileNameNo;
    $imgPath = WWW_ROOT . 'img' . DS . $this->imageDir . DS . $newFileName . '.' . $ext;
    //一時ファイルから動かす。
    //サムネイル生成
    $Imageresizer = new Imageresizer();
    $Imageresizer->resize($data['tmp_name'], $imgPath, $width, $height);
    //一時ファイルを消し去ってしまう。
    return $newFileName . '.' . $ext;
  }

  /**
  * 画像を削除する。
  *
  * @params string $imgKey
  * @params array $data
  * @return bool
  * @access protected
  */
  protected function _deletePostImage($imgKey, $data){
    //現在のデータを確認する。
    $scheduleData = $this->find('first', array(
      'conditions' => array(
        'Schedule.id' => $data['Schedule']['id']
      ),
      'field' => array(
        "Schedule.$imgKey" //メモリ節約の為必要なフィールドのみ取得
      )));
      //削除処理
      if (!empty($scheduleData['Schedule']["$imgKey"])) {
        $delFileName = WWW_ROOT . 'img' . DS . $this->imageDir . DS . $scheduleData['Schedule']["$imgKey"];
        if (file_exists($delFileName)) {
          @unlink($delFileName);
          return true;
        }else{
          return false;
        }
      }
    }


  /**
  * 並び順を変更する
  * @param string	$id
  * @param int			$offset
  * @param array		$conditions
  * @return boolean
  */
	public function changeSort($id, $offset, $conditions = array()) {

    //バリデーション解除
    unset($this->validate);

		if ($conditions) {
			$_conditions = $conditions;
		} else {
			$_conditions = array();
		}

		// 一時的にキャッシュをOFFする
		$this->cacheQueries = false;

		$current = $this->find('first', array(
			'conditions' => array($this->alias . '.id' => $id),
			'fields' => array($this->alias . '.id', $this->alias . '.sort')
		));

		// 変更相手のデータを取得
		if ($offset > 0) { // DOWN
			$order = array($this->alias . '.sort');
			$limit = $offset;
			$conditions[$this->alias . '.sort >'] = $current[$this->alias]['sort'];
		} elseif ($offset < 0) { // UP
			$order = array($this->alias . '.sort DESC');
			$limit = $offset * -1;
			$conditions[$this->alias . '.sort <'] = $current[$this->alias]['sort'];
		} else {
			return true;
		}

		$conditions = array_merge($conditions, $_conditions);
		$target = $this->find('all', array(
			'conditions' => $conditions,
			'fields' => array($this->alias . '.id', $this->alias . '.sort'),
			'order' => $order,
			'limit' => $limit,
			'recursive' => -1
		));

		if (!isset($target[count($target) - 1])) {
      //ここで落ちる
			return false;
		}

		$currentSort = $current[$this->alias]['sort'];
		$targetSort = $target[count($target) - 1][$this->alias]['sort'];

		// current から target までのデータをsortで範囲指定して取得
		$conditions = array();
		if ($offset > 0) { // DOWN
			$conditions[$this->alias . '.sort >='] = $currentSort;
			$conditions[$this->alias . '.sort <='] = $targetSort;
		} elseif ($offset < 0) { // UP
			$conditions[$this->alias . '.sort <='] = $currentSort;
			$conditions[$this->alias . '.sort >='] = $targetSort;
		}

		$conditions = array_merge($conditions, $_conditions);
		$datas = $this->find('all', array(
			'conditions' => $conditions,
			'fields' => array($this->alias . '.id', $this->alias . '.sort'),
			'order' => $order,
			'recursive' => -1
		));

		// 全てのデータを更新
		foreach ($datas as $data) {
			if ($data[$this->alias]['sort'] == $currentSort) {
				$data[$this->alias]['sort'] = $targetSort;
			} else {
				if ($offset > 0) {
					$data[$this->alias]['sort'] --;
				} elseif ($offset < 0) {
					$data[$this->alias]['sort'] ++;
				}
			}
			if (!$this->save($data, false)) {
				return false;
			}
		}
		return true;
	}
}
