<?php
/*
* スケジュールプラグイン
* ヘルパー
*
* PHP 5.4.x
*
* @copyright    Copyright (c) hiniarata co.ltd
* @link         https://hiniarata.jp
* @package      Schedule Plugin Project
* @since        ver.0.9.0
*/
/* Model */
App::import('Model', 'Schedule.Schedule');
App::import('Model', 'Schedule.ScheduleCategory');
App::import('Model', 'Schedule.ScheduleHoliday');
App::import('Model', 'Schedule.ScheduleHolidaySetting');
App::uses('AppHelper', 'View/Helper');

class ScheduleBaserHelper extends AppHelper {

  /**
  * ヘルパー
  *
  * @var array
  */
  public $helpers = array('BcBaser', 'Html');


  /**
  * 新着スケジュール一覧を表示する。
  *
  * @param $num 取得件数
  * @param $options オプション
  * @return array()
  */
  public function schedules($num = 5, $options = array()){
    $options = array_merge(array(
      'category' => null,
      'template' => null
		), $options);

		$url = array('admin' => false, 'plugin' => 'schedule', 'controller' => 'schedules', 'action' => 'schedules');

		$settings = Configure::read('BcAgent');
		foreach ($settings as $key => $setting) {
			if (isset($options[$key])) {
				$agentOn = $options[$key];
				unset($options[$key]);
			} else {
				$agentOn = (Configure::read('BcRequest.agent') == $key);
			}
			if ($agentOn) {
				$url['prefix'] = $setting['prefix'];
				break;
			}
		}

		echo $this->requestAction($url, array('return', 'pass' => array($num), 'named' => $options));
  }

  /**
  * 新着スケジュールデータを取得する。
  *
  * @param $num 取得件数
  * @param $options オプション
  * @return array()
  */
  public function getScheduleNewDatas($num = null, $options = array()){
    //取得件数
    if (empty($num)) {
      $num = 5;
    }
    //オプション値
    $options = array_merge(array(
     'eyecatch' => null, //1〜4でどれを使うかを指定する。
     'category' => null, //特定のカテゴリに属したものだけを表示する
     'order' => null
    ), $options);

    //カテゴリオプションの確認
    if (!empty($options['category'])) {
      $conditions[] = array('Schedule.schedule_category_id' => $options['category']);
    }
    //オーダーオプションの確認
    if (!empty($options['order'])) {
      $order = $options['order'];
    } else {
      $order = 'DESC';
    }

    //対象データを取得
    $scheduleModel = new Schedule();
    //データの取得
    $schedules = $scheduleModel->find('all', array(
      'conditions' => array('Schedule.publish' => 1),
      'order' => array('Schedule.id' => $order),
      'limit' => $num
    ));

    //アイキャッチオプションの確認
    if (!empty($options['eyecatch'])) {
      for($i=0; $i<count($schedules); $i++){
        if (!empty($schedules[$i]['Schedule']['image_'.$options['eyecatch']])) {
          $schedules[$i]['Schedule']['eyecatch'] = $schedules[$i]['Schedule']['image_'.$options['eyecatch']];
        } else {
          $schedules[$i]['Schedule']['eyecatch'] = '';
        }
      }

    //オプション指定がない場合、１〜４の順番で上書きしていく。（１が最優先）
    } else {
      for($i=0; $i<count($schedules); $i++){
        $schedules[$i]['Schedule']['eyecatch'] = '';
        for ($c=4; $c>0; $c--) {
          if (!empty($schedules[$i]['Schedule']['image_'.$c])) {
            $schedules[$i]['Schedule']['eyecatch'] = $schedules[$i]['Schedule']['image_'.$c];
          }
        }
      }
    }
    return $schedules;
  }


  /**
  * スケジュールデータを取得する。
  *
  * @param $year 表示したい年
  * @param $month 表示したい月
  * @param $options オプション
  * @return array()
  */
  public function getScheduleDatas($year = null, $month = null, $options = array()){
    //オプション値
    $options = array_merge(array(
     'eyecatch' => null, //1〜4でどれを使うかを指定する。
     'category' => null, //特定のカテゴリに属したものだけを表示する
     'order' => null
    ), $options);
    //表示年月の初期値
    if (empty($year)) {
      $year = date("Y");
    }
    if (empty($month)) {
      $month = date("m");
    } else {
      //月は01、02など先頭に0がつくように調整する。
      $month = sprintf("%02d",$month);
    }

    //対象データを取得
    $scheduleModel = new Schedule();
    //翌月の計算
    if ($month == 12) {
        $nextYear = $year+1;
        $nextMonth = 1;
    } else {
        $nextMonth = $month+1;
        $nextYear = $year;
    }
    //検索条件
    $conditions = array(
      'Schedule.publish' => 1,
      'Schedule.begin_date <' => $nextYear .'-'. $nextMonth .'-'. '01' .' 00:00:00', //翌月1日までに始まっており、
      'Schedule.end_date >=' =>  $year .'-'. $month .'-'. '01' .' 00:00:00' //かつ当月1日に終わっていないもの。
    );
    //カテゴリオプションの確認
    if (!empty($options['category'])) {
      $conditions[] = array('Schedule.schedule_category_id' => $options['category']);
    }
    //オーダーオプションの確認
    if (!empty($options['order'])) {
      $order = $options['order'];
    } else {
      $order = 'ASC';
    }

    //データの取得
    $schedules = $scheduleModel->find('all', array(
      'conditions' => $conditions,
      'order' => array('Schedule.sort' => $order)
    ));

    //アイキャッチオプションの確認
    if (!empty($options['eyecatch'])) {
      for($i=0; $i<count($options); $i++){
        if (!empty($schedules[$i]['Schedule']['image_'.$options['eyecatch']])) {
          $schedules[$i]['Schedule']['eyecatch'] = $schedules[$i]['Schedule']['image_'.$options['eyecatch']];
        } else {
          $schedules[$i]['Schedule']['eyecatch'] = '';
        }
      }

    //オプション指定がない場合、１〜４の順番で上書きしていく。（１が最優先）
    } else {
      for($c=0; $c<count($options); $c++){
        $schedules[$c]['Schedule']['eyecatch'] = '';
        for ($i=4; $i>0; $i--) {
          if (!empty($schedules[$c]['Schedule']['image_'.$i])) {
            $schedules[$c]['Schedule']['eyecatch'] = $schedules[$c]['Schedule']['image_'.$i];
          }
        }
      }
    }
    return $schedules;
  }


  /**
   * スケジュールカレンダーを出力する。
   *
   * @param $year 表示したい年
   * @param $month 表示したい月
   * @param $options オプション
   * @return array()
   */
   public function scheduleCalendar($year = null, $month = null, $options = array()){
     echo $this->getScheduleCalendar($year, $month, $options);
   }


  /**
  * スケジュールカレンダーを取得する。
  *
  * @param $year 表示したい年
  * @param $month 表示したい月
  * @param $options オプション
  * @return array()
  */
  public function getScheduleCalendar($year = null, $month = null, $options = array()){
    //オプション値
    $options = array_merge(array(
     'title' => null, //カレンダー内にスケジュールのタイトルを表示するかどうか。
     'class' => null, //テーブルにつけるクラス名を指定できる。
     'id' => null, //テーブルにつけるid
     'category' => null //特定のカテゴリに属したものだけを表示する
    ), $options);
    //表示年月の初期値
    if (empty($year)) {
      $year = date("Y");
    }
    if (empty($month)) {
      $month = date("m");
    } else {
      //月は01、02など先頭に0がつくように調整する。
      $month = sprintf("%02d",$month);
    }

    //対象データを取得
    $scheduleModel = new Schedule();
    //翌月の計算
    if ($month == 12) {
        $nextYear = $year+1;
        $nextMonth = 1;
    } else {
        $nextMonth = $month+1;
        $nextYear = $year;
    }
    //検索条件
    $conditions = array(
      'Schedule.publish' => 1,
      'Schedule.begin_date <' => $nextYear .'-'. $nextMonth .'-'. '01' .' 00:00:00', //翌月1日までに始まっており、
      'Schedule.end_date >=' =>  $year .'-'. $month .'-'. '01' .' 00:00:00' //かつ当月1日に終わっていないもの。
    );
    //カテゴリオプションの確認
    if (!empty($options['category'])) {
      $conditions[] = array('Schedule.schedule_category_id' => $options['category']);
    }
    //データの取得
    $schedules = $scheduleModel->find('all', array(
      'conditions' => $conditions,
      'order' => array('Schedule.sort' => 'ASC')
    ));
    //echo $year.$month;exit();
    //カレンダーデータ生成
    return $this->makeCalender($year, $month, $schedules, $options);
  }



  /**
  * スケジュールカレンダーを取得する。
  *
  * @param $year 表示したい年
  * @param $month 表示したい月
  * @param $options オプション
  * @return array()
  */
  public function makeCalender($year, $month, $schedules = null, $options = array()){
      //ヘッダー部分の曜日表示
      $days = array('日', '月', '火', '水', '木', '金', '土');
      //本日を取得（20120512）
      $today = date('Ymd');
      //1行ごとの順番（1〜7）と、月全体の順番（1〜31（最大））
      $num_row = 1;
      $num_day = 1;
      //月の始まる曜日から、前月の空白分を取得
      $num_blank = date('w', mktime(0, 0, 0, $month, 1, $year));
      //月の合計日
      $total = $month == 2 ? ($year % 4 ? 28 : ($year % 100 ? 29 : ($year % 400 ? 28 : 29))) : (($month - 1) % 7 % 2 ? 30 : 31);

      //オプション
      if(!empty($options['class'])){
        $class = ' ' . $options['class'];
      } else {
        $class = '';
      }
      if(!empty($options['id'])){
        $id = 'id="'. $options['id'].'"';
      } else {
        $id = '';
      }

      //カレンダーのヘッダー部分
      $calendar = '<table class="scheduleCalendar'.$class.'" '.$id.'>';
      $calendar .= '<thead><tr>';

      while ($num_row <= count($days)) {
          //曜日クラスを付与する。
          if ($num_row == 1) {
            $classDay = ' class="sunday"';
          } elseif ($num_row == 7) {
            $classDay = ' class="satday"';
          } else {
            $classDay = '';
          }
          $calendar .= '<th'.$classDay.'>'.$days[$num_row-1].'</th>';
          $num_row++;
      }
      $calendar .= '</tr></thead>';

      //カレンダーの本体部分
      $calendar .= '<tbody><tr>';
      $num_row = 1;

      //前月の空白分のセルを生成
      while ($num_blank > 0) {
          $label = '';
          $label = ($num_row == 1) ? $label.' sun' : $label;
          $calendar .= '<td class="'.$label.'"></td>';
          $num_blank--;
          $num_row++;
      }

      //１日ずつ繰り返し処理開始
      //1日〜31日（最大）までのセルを生成
      while ($num_day <= $total) {
          //「$label」は要素のクラス名
          $label = '';
          $content = $num_day; //日付（もし投稿がある日なら以後の処理内で上書きする）

          //イベントの配列を走査するためのキーとして、「20120512」の形式で文字列を生成
          $key = $year.sprintf('%02d', $month).sprintf('%02d', $num_day);

          //左端なら日曜日、右端なら土曜日のためのラベル（クラス）を付加
          $label = ($num_row == 1) ? $label.' sun' : $label;
          $label = ($num_row == 7) ? $label.' sat' : $label;
          //本日の場合もクラスを付加
          $label = ($key == $today) ? $label.' today' : $label;

          //イベントの配列があれば、配列を走査
          //TODO: 登録件数が少ないのでこの方法でも出来るが、もっと効率的な方法がある気がする・・・。
          if (!empty($schedules)) {
            foreach ($schedules as $schedule) {
              //登録されているスケジュール期間を確認する。
              $begin = date("Ymd", strtotime($schedule['Schedule']['begin_date']));
              $end = date("Ymd", strtotime($schedule['Schedule']['end_date']));

              //期間内ならリンクを設定する。
              if ($begin <= $key && $end >= $key) {
                //まずはクラスの付与
                $label = " scheduleOn";

                //タイトル出力がtrueなら、内容を付与する。
                if($options['title']){
                  //リンク先の生成
                  $url = array(
                    'plugin' => 'schedule',
                    'controller' => 'schedules',
                    'action' => 'view',
                    $schedule['Schedule']['id']
                  );
                  $link = $this->Html->link(h($schedule['Schedule']['title']), $url);
                  $content .= "<div class='scheduleLink'>". $link ."</div>";
                //タイトル表示不可であれば日付にリンクをはる。
                } else {
                  //リンク先の生成
                  $url = array(
                    'plugin' => 'schedule',
                    'controller' => 'schedules',
                    'action' => 'day',
                    $year,
                    sprintf('%02d', $month),
                    sprintf('%02d', $num_day)
                  );
                  $link = $this->Html->link($num_day, $url);
                  $content = $link;
                }
              }
            }
          }

          //その日のイベントは、$contentに出力される。
          $calendar .= '<td class="'.$label.'">';
          $calendar .= $content;
          $calendar .= '</td>';
          $num_day++;
          $num_row++;

          //1行の終わり（7日目）ごとに、新しい行を開始
          if ($num_row > 7) {
              $calendar .= '</tr><tr>';
              $num_row = 1;
          }
      }

      //月の最終日以降の空白分のセルを生成
      while ($num_row > 1 && $num_row <= 7) {
          $label = '';
          $label = ($num_row == 7) ? $label.' sat' : $label;
          $calendar .= '<td class="'.$label.'"></td>';
          $num_row++;
      }
      //カレンダーの終了
      $calendar .= '</tr></tbody></table>';
      //文字列として生成したカレンダーを返す
      return $calendar;
    }



  /**
   * コンテンツテンプレートを取得（フォルダ単位）
   *
   * @return array()
   */
    public function getScheduleTemplates() {
      $templatesPathes = array_merge(App::path('View', 'Schedule'), App::path('View'));
      if ($this->BcBaser->siteConfig['theme']) {
        array_unshift($templatesPathes, WWW_ROOT . 'theme' . DS . $this->BcBaser->siteConfig['theme'] . DS);
      }

      $_templates = array();
      foreach ($templatesPathes as $templatePath) {
        $templatePath .= 'Schedules' . DS;
        $folder = new Folder($templatePath);
        $files = $folder->read(true, true);
        $foler = null;
        if ($files[0]) {
          if ($_templates) {
            $_templates = am($_templates, $files[0]);
          } else {
            $_templates = $files[0];
          }
        }
      }

      $excludes = Configure::read('BcAgent');
      $excludes = Hash::extract($excludes, '{s}.prefix');
      $templates = array();
      foreach ($_templates as $template) {
        if (!in_array($template, $excludes)) {
          $templates[$template] = $template;
        }
      }
      return $templates;
    }


  /**
   * 休日カレンダーを出力する。
   *
   * @param $year 表示したい年
   * @param $month 表示したい月
   * @return array()
   */
   public function holidayCalendar($year = null, $month = null, $options = array()){
     echo $this->getHolidayCalendar($year, $month, $options);
   }


   /**
    * 休日カレンダーを取得する。
    *
    * @param $year 表示したい年
    * @param $month 表示したい月
    * @return array()
    */
    public function getHolidayCalendar($year = null, $month = null, $options = array()){
      //オプション値
      $options = array_merge(array(
       'holiday' => null, //休日に表示したい文字
       'business_day' => null, //営業日に表示したい文字
       'class' => null, //テーブルにつけるクラス名を指定できる。
       'id' => null, //テーブルにつけるid
      ), $options);
      //年月が無ければ現在の表示
      if (empty($year)) {
        $year = date("Y");
      }
      if (empty($month)) {
        $month = date("m");
      } else {
        //月は01、02など先頭に0がつくように調整する。
        $month = sprintf("%02d",$month);
      }

      //基本設定値と休日の登録具合を取得する。
      //取得したデータは、日付と曜日がキーになる配列を組む。
      $holidayModel = new ScheduleHoliday();
      $settingModel = new ScheduleHolidaySetting();
      $holidayDatas = $holidayModel->find('first', array('conditions' => array(
        'first_day' => date("Y-m-d", strtotime($year.'-'.$month.'-01'))
      )));
      //各月の設定
      $checkHoliday = array(); //設定でチェックされた日
      if (!empty($holidayDatas['ScheduleHoliday']['holidays'])) {
        //アンシリアライズ
        $holidayDatas['ScheduleHoliday']['holidays'] = unserialize($holidayDatas['ScheduleHoliday']['holidays']);
        //チェックされた日をキーにする配列を組む
        foreach ($holidayDatas['ScheduleHoliday']['holidays'] as $data) {
          $checkHoliday[$data] = 'on';
        }
      }
      //毎月の基本設定（第○週ごとに設定されている）
      $setting = $settingModel->findById(1);
      $weekHoliday = array(); //週ごとの休日曜日
      //アンシリアライズ
      if (!empty($setting['ScheduleHolidaySetting']['week1'])) {
        $setting['ScheduleHolidaySetting']['week1'] = unserialize($setting['ScheduleHolidaySetting']['week1']);
        //曜日番号（1〜７）をキーとする休日設定の配列をつくる
        foreach($setting['ScheduleHolidaySetting']['week1'] as $data){
          $weekHoliday['week1'][$data] = 'on';
        }
      }
      if (!empty($setting['ScheduleHolidaySetting']['week2'])) {
        $setting['ScheduleHolidaySetting']['week2'] = unserialize($setting['ScheduleHolidaySetting']['week2']);
        foreach($setting['ScheduleHolidaySetting']['week2'] as $data){
          $weekHoliday['week2'][$data] = 'on';
        }
      }
      if (!empty($setting['ScheduleHolidaySetting']['week3'])) {
        $setting['ScheduleHolidaySetting']['week3'] = unserialize($setting['ScheduleHolidaySetting']['week3']);
        foreach($setting['ScheduleHolidaySetting']['week3'] as $data){
          $weekHoliday['week3'][$data] = 'on';
        }
      }
      if (!empty($setting['ScheduleHolidaySetting']['week4'])) {
        $setting['ScheduleHolidaySetting']['week4'] = unserialize($setting['ScheduleHolidaySetting']['week4']);
        foreach($setting['ScheduleHolidaySetting']['week4'] as $data){
          $weekHoliday['week4'][$data] = 'on';
        }
      }
      if (!empty($setting['ScheduleHolidaySetting']['week5'])) {
        $setting['ScheduleHolidaySetting']['week5'] = unserialize($setting['ScheduleHolidaySetting']['week5']);
        foreach($setting['ScheduleHolidaySetting']['week5'] as $data){
          $weekHoliday['week5'][$data] = 'on';
        }
      }
      if (!empty($setting['ScheduleHolidaySetting']['week6'])) {
        $setting['ScheduleHolidaySetting']['week6'] = unserialize($setting['ScheduleHolidaySetting']['week6']);
        foreach($setting['ScheduleHolidaySetting']['week6'] as $data){
          $weekHoliday['week6'][$data] = 'on';
        }
      }

      //カレンダーの生成
      //ヘッダー部分の曜日表示
      $days = array('日', '月', '火', '水', '木', '金', '土');
      //本日を取得（20120512）
      $today = date('Ymd');
      //1行ごとの順番（1〜7）と、月全体の順番（1〜31（最大））
      $num_row = 1;
      $num_day = 1;
      //月の始まる曜日から、前月の空白分を取得
      $num_blank = date('w', mktime(0, 0, 0, $month, 1, $year));
      //月の合計日
      $total = $month == 2 ? ($year % 4 ? 28 : ($year % 100 ? 29 : ($year % 400 ? 28 : 29))) : (($month - 1) % 7 % 2 ? 30 : 31);
      //何週目かのフラグ。
      $weekNo = 1; //trの閉じタグで増加する。

      //オプション
      if(!empty($options['class'])){
        $class = ' ' . $options['class'];
      } else {
        $class = '';
      }
      if(!empty($options['id'])){
        $id = 'id="'. $options['id'].'"';
      } else {
        $id = '';
      }

      //カレンダーのヘッダー部分
      $calendar = '<table class="scheduleCalendar'.$class.'" '.$id.'>';
      $calendar .= '<thead><tr>';

      while ($num_row <= count($days)) {
          //曜日クラスを付与する。
          if ($num_row == 1) {
            $classDay = ' class="sunday"';
          } elseif ($num_row == 7) {
            $classDay = ' class="satday"';
          } else {
            $classDay = '';
          }
          $calendar .= '<th'.$classDay.'>'.$days[$num_row-1].'</th>';
          $num_row++;
      }
      $calendar .= '</tr></thead>';

      //カレンダーの本体部分
      $calendar .= '<tbody><tr>';
      $num_row = 1;

      //前月の空白分のセルを生成
      while ($num_blank > 0) {
          $label = '';
          $label = ($num_row == 1) ? $label.' sun' : $label;
          $calendar .= '<td class="'.$label.'"></td>';
          $num_blank--;
          $num_row++;
      }

      //１日ずつ繰り返し処理開始
      //1日〜31日（最大）までのセルを生成
      while ($num_day <= $total) {
          //「$label」は要素のクラス名
          $label = '';
          $content = $num_day; //日付（もし投稿がある日なら以後の処理内で上書きする）

          //イベントの配列を走査するためのキーとして、「20120512」の形式で文字列を生成
          $key = $year.sprintf('%02d', $month).sprintf('%02d', $num_day);

          //左端なら日曜日、右端なら土曜日のためのラベル（クラス）を付加
          $label = ($num_row == 1) ? $label.' sun' : $label;
          $label = ($num_row == 7) ? $label.' sat' : $label;
          //本日の場合もクラスを付加
          $label = ($key == $today) ? $label.' today' : $label;
          //カレンダーの中身（日付等）
          $holidayValue = $num_day;

          //休日の設定がなされているかどうか。
          //営業日か休日かでクラスを割り振る。
          //まず各月の休日設定があるかどうかを確認する。あれば利用し、なければ基本設定を利用する。
          if (!empty($holidayDatas)) {
            if (!empty($checkHoliday[$num_day])) {
              $label = ' holiday';
              if (!empty($options['holiday'])) {
                $holidayValue .= '<div class="holidayCalendarInner-holiday">'.$options['holiday'].'</div>';
              }
            } else {
              $label = ' businessDay';
              if (!empty($options['business_day'])) {
                $holidayValue .= '<div class="holidayCalendarInner-business">'.$options['business_day'].'</div>';
              }
            }

          //基本設定を利用
          } else {
            if (!empty($weekHoliday['week'.$weekNo][$num_row])) { //週番号と曜日番号で判定する。
              $label = ' holiday';
              $holidayValue .= '<div class="holidayCalendarInner-holiday">'.$options['holiday'].'</div>';
            } else {
              $label = ' businessDay';
              $holidayValue .= '<div class="holidayCalendarInner-business">'.$options['business_day'].'</div>';
            }
          }

          $calendar .= '<td class="'.$label.'">'.$holidayValue.'</td>';
          $num_day++;
          $num_row++;

          //1行の終わり（7日目）ごとに、新しい行を開始
          if ($num_row > 7) {
              $calendar .= '</tr><tr>';
              $num_row = 1;
              $weekNo++;
          }
      }

      //月の最終日以降の空白分のセルを生成
      while ($num_row > 1 && $num_row <= 7) {
          $label = '';
          $label = ($num_row == 7) ? $label.' sat' : $label;
          $calendar .= '<td class="'.$label.'"></td>';
          $num_row++;
      }
      //カレンダーの終了
      $calendar .= '</tr></tbody></table>';
      return $calendar;
    }
}
