<?php
/*
* スケジュールプラグイン
* ヘルパー
*
* PHP 5.4.x
*
* @copyright    Copyright (c) hiniarata co.ltd
* @link         https://hiniarata.jp
* @package      Schedule Plugin Project
* @since        ver.0.9.0
*/
App::uses('AppHelper', 'View/Helper');

class ScheduleHelper extends AppHelper {

  /**
  * ヘルパー
  *
  * @var array
  */
  public $helpers = array('Html', 'BcBaser');

  /**
  * スケジュールのタイトルを出力する
  *
  * @param $schedule スケジュールデータ
  * @param $link リンクの有無
  * @param $options リンクオプション
  * @return void
  */
  public function title($schedule, $link = false, $options = array()){
    echo $this->getTitle($schedule, $link, $options);
  }

  /**
  * スケジュールのタイトルを出力する
  *
  * @param $schedule スケジュールデータ
  * @param $link リンクの有無
  * @param $options リンクオプション
  * @return text
  */
  public function getTitle($schedule, $link = false, $options = array()){
    if ($link) {
      $url = array(
        'plugin' => 'schedule',
        'controller' => 'schedules',
        'action' => 'view',
        $schedule['Schedule']['id']
      );
      echo $this->Html->link($schedule['Schedule']['title'], $url, $options);
    } else {
      return $schedule['Schedule']['title'];
    }

  }

  /**
  * スケジュールの内容を出力する
  *
  * @param $schedule スケジュールデータ
  * @return void
  */
  public function content($schedule){
    echo $this->getContent($schedule);
  }

  /**
  * スケジュールの内容を出力する
  *
  * @param $schedule スケジュールデータ
  * @return text
  */
  public function getContent($schedule){
    return $schedule['Schedule']['text'];
  }

  /**
  * スケジュールのカテゴリ名を出力する
  *
  * @param $schedule スケジュールデータ
  * @return void
  */
  public function category($schedule){
    echo $this->getCategory($schedule);
  }

  /**
  * スケジュールの内容を出力する
  *
  * @param $schedule スケジュールデータ
  * @return text
  */
  public function getCategory($schedule){
    if (!empty($schedule['ScheduleCategory']['title'])) {
      return $schedule['ScheduleCategory']['title'];
    } else {
      return "";
    }
  }

  /**
  * スケジュールの開始日を出力する
  *
  * @param $schedule スケジュールデータ
  * @param $format 日付形式
  * @return void
  */
  public function beginDate($schedule, $format = 'Y/m/d H:i'){
    echo $this->getBeginDate($schedule, $format);
  }

  /**
  * スケジュールの開始日を取得する
  *
  * @param $schedule スケジュールデータ
  * @param $format 日付形式
  * @return text
  */
  public function getBeginDate($schedule, $format = 'Y/m/d H:i'){
    return date($format, strtotime($schedule['Schedule']['begin_date']));
  }

  /**
  * スケジュールの終了日を出力する
  *
  * @param $schedule スケジュールデータ
  * @param $format 日付形式
  * @return void
  */
  public function endDate($schedule, $format = 'Y/m/d H:i'){
    echo $this->getEndDate($schedule, $format);
  }

  /**
  * スケジュールの終了日を取得する
  *
  * @param $schedule スケジュールデータ
  * @param $format 日付形式
  * @return text
  */
  public function getEndDate($schedule, $format = 'Y/m/d H:i'){
    return date($format, strtotime($schedule['Schedule']['end_date']));
  }

  /**
  * スケジュールの画像を出力する
  *
  * @param array $schedule スケジュールデータ
  * @param int $number 1〜4の画像番号
  * @return void
  */
  public function image($schedule, $number){
    $imageName = $this->getImageName($schedule, $number);
    if (!empty($imageName)) {
      echo $this->Html->image("schedule_plugin/". $imageName);
    }
  }

  /**
  * スケジュールの画像名をする
  *
  * @param array $schedule スケジュールデータ
  * @param int $number 1〜4の画像番号
  * @return void
  */
  public function getImageName($schedule, $number){
    switch ($number) {
      case 1:
        return $schedule['Schedule']['image_1'];
        break;
      case 2:
        return $schedule['Schedule']['image_2'];
        break;
      case 3:
        return $schedule['Schedule']['image_3'];
        break;
      case 4:
        return $schedule['Schedule']['image_4'];
        break;
      default:
        return "";
        break;
    }
  }

  /**
  * スケジュールの地図を出力する
  *
  * @param array $schedule スケジュールデータ
  * @return void
  */
  public function map($schedule, $width = "480px", $height = "300px"){
    if ($schedule['Schedule']['map'] == 1) { // 1:利用する
      echo $this->getMap($schedule, $width, $height);
    }
  }

  /**
  * スケジュールの地図を取得する
  *
  * @param array $schedule スケジュールデータ
  * @return void
  */
  public function getMap($schedule, $width, $height){
    //座標が空の場合
    if (empty($schedule['Schedule']['lat']) || empty($schedule['Schedule']['lng'])) {
      return false;

    //座標が入っている場合。
    } else{
      $map = '<script type="text/javascript" src="http://maps.google.com/maps/api/js?sensor=false"></script>';
      $map .= '<div id="scheduleGoogleMap" style="width:'.$width.'; height:'.$height.';"></div>';
      $map .= '<script type="text/javascript">
            // 地図
            var map = new google.maps.Map(document.getElementById("scheduleGoogleMap"), {
              zoom: 16,
              center: new google.maps.LatLng('.$schedule['Schedule']['lat'].', '.$schedule['Schedule']['lng'].'),
              scrollwheel: false,
              mapTypeId: google.maps.MapTypeId.ROADMAP
            });
            // マーカー
            new google.maps.Marker({
              position: new google.maps.LatLng('.$schedule['Schedule']['lat'].', '.$schedule['Schedule']['lng'].'),
              map: map,
            });
            </script>';
      return $map;
    }
  }

}
